@@ -1,5 +1,5 @@
 # SystemC - SystemC Perl Interface
-# $Id: AutoTrace.pm,v 1.14 2001/11/16 15:01:41 wsnyder Exp $
+# $Id: AutoTrace.pm,v 1.21 2002/03/11 15:52:09 wsnyder Exp $
 # Author: Wilson Snyder <wsnyder@wsnyder.org>
 ######################################################################
 #
@@ -25,9 +25,11 @@ package SystemC::Netlist::AutoTrace;
 use File::Basename;
 
 use SystemC::Netlist::Module;
-$VERSION = '1.000';
+$VERSION = '1.100';
 use strict;
 
+use vars qw ($Setup_Ident_Code);	# Local use for recursion only
+
 ######################################################################
 #### Automatics (Preprocessing)
 
@@ -46,6 +48,7 @@ sub _write_autotrace {
 
     # Flatten out all hiearchy under this into a array of signal information
     my @tracevars = ();
+    local $Setup_Ident_Code = 0;
     _tracer_setup($self,\@tracevars,
 		  ($self->_autotrace() eq 'recurse'),
 		  );
@@ -80,7 +83,9 @@ sub _tracer_setup {
     my $level = shift || 1;
     my $nethier = shift || "t";
     my $modhier = shift || "";
+    my $upper_codes_ref = shift || {};
 
+    my %our_codes;
     foreach my $netref ($modref->nets_sorted()) {
 	next if ($netref->name =~ /^_/);	# Skip leading _ signals
 	my $ignore = 0;
@@ -96,7 +101,8 @@ sub _tracer_setup {
 	if ($scbv) {
 	    $accessor .= "(((uint32_t*)";
 	}
-	$accessor .= $nethier.'->'.$netref->name;
+	#$accessor .= $nethier.'->'.$netref->name;
+	$accessor .= 'ts->'.$netref->name;
 	if ($netref->array) {
 	    $accessor .= "[i]";
 	}
@@ -123,7 +129,10 @@ sub _tracer_setup {
 	if (!$ignore) {
 	    $code_inc = (int($netref->width()/32) + 1);
 	}
-	push @{$tvarref}, {
+	my $identical = $upper_codes_ref->{$netref->name};
+
+	# Store info for this var
+	my $tref = {
 	    modref => $modref,
 	    netref => $netref,
 	    level  => $level,
@@ -131,15 +140,39 @@ sub _tracer_setup {
 	    modhier => $modhier,
 	    code_inc => $code_inc,
 	    ignore => $ignore,
+	    identical => $identical,
+	    identical_code => ($identical || 0),
 	    accessor => $accessor,
 	};
+	push @{$tvarref}, $tref;
+	$our_codes{$netref->name} = $tref if $recurse;
     }
     if ($recurse) {
 	foreach my $cellref ($modref->cells_sorted()) {
+	    my %dup_codes = ();
+	    if (!$modref->netlist->{sp_trace_duplicates}) {
+		foreach my $pinref ($cellref->pins_sorted) {
+		    if ($pinref->net && $pinref->port && $pinref->port->net
+			&& $pinref->port->net->name eq $pinref->net->name
+			&& $pinref->port->net->array eq $pinref->net->array
+			&& $pinref->port->net->type eq $pinref->net->type
+			&& $pinref->port->net->msb eq $pinref->net->msb
+			&& $pinref->port->net->lsb eq $pinref->net->lsb
+			) { # Then, it's the same signal.
+			#print "PIN ",$cellref->name," XX ", $pinref->name,"\n";
+			my $ourref = $our_codes{$pinref->net->name};
+			$ourref->{identical_code} ||= ++$Setup_Ident_Code;
+			my $ourcode = $ourref->{identical_code};
+			$ourref->{identical_child} = $ourcode;	# May be more then one
+			$dup_codes{$pinref->port->net->name} = $ourcode;
+		    }
+		}
+	    }
 	    _tracer_setup($cellref->submod, $tvarref,
 			  $recurse,
 			  $level+1, $nethier."->".$cellref->name,
 			  $modhier.".".$cellref->name,
+			  \%dup_codes,
 			  );
 	}
     }
@@ -155,16 +188,21 @@ sub _write_tracer_trace {
 	 "{\n",
 	 "    if(0 && options) {}  // Prevent unused\n",
 	 "    tfp->spTrace()->addCallback (&${mod}::traceInit, &${mod}::traceChange, this);\n",);
-    $fileref->print ("    if (levels > 0) {\n",);
+    my $cmt = "";
+    if ($self->_autotrace() eq 'recurse') {
+	$fileref->print ("    // Inline child recursion, so don't need:\n");
+	$cmt = "//";
+    }
+    $fileref->print ("    ${cmt}if (levels > 0) {\n",);
     foreach my $cellref ($self->cells_sorted) {
 	my $name = $cellref->name;
 	(my $namenobra = $name) =~ tr/\[\]/()/;
 	if ($cellref->submod->_autotrace()) {
-	    $fileref->printf ("        this->${name}->trace (tfp, levels-1, options);  // Is-a %s\n",
+	    $fileref->printf ("    ${cmt}    this->${name}->trace (tfp, levels-1, options);  // Is-a %s\n",
 				     $cellref->submod->name);
 	}
     }
-    $fileref->print ("    }\n",
+    $fileref->print ("    ${cmt}}\n",
 		     "}\n",);
 }
 
@@ -176,6 +214,7 @@ sub _write_tracer_init {
     my $mod = $modref->name;
     $fileref->print("void ${mod}::traceInit (SpTraceVcd* vcdp, void* userthis, uint32_t code)\n");
     $fileref->print("{\n");
+    $fileref->printf("  int _identcode[%d];\n", $Setup_Ident_Code+1) if $Setup_Ident_Code;
     $fileref->print("  // Callback from vcd->open()\n");
     $fileref->print("  if (0 && vcdp && userthis && code) {}  // Prevent unused\n");
     if ($#tracevars >= 0) {
@@ -183,21 +222,38 @@ sub _write_tracer_init {
 	$fileref->print("  ${mod}* t=(${mod}*)userthis;\n");
 	$fileref->print("  string prefix = t->name();\n");
     }
+    $fileref->printf("  {\n");
     my $last_modhier = undef;
     foreach my $tref (@tracevars) {
 	my $modref = $tref->{modref};
 	my $netref = $tref->{netref};
 	my $indent = "  "x$tref->{level};
 	my $accessor = $tref->{accessor};
+	# Scope to correct parent module
 	if (!defined $last_modhier || $last_modhier ne $tref->{modhier}) {
 	    $last_modhier = $tref->{modhier};
-	    $fileref->printf("${indent}vcdp->module(prefix+\"%s\");  // Is-a %s\n"
+	    $fileref->printf("${indent}"."}{\n");
+	    $fileref->printf("${indent}"."vcdp->module(prefix+\"%s\");  // Is-a %s\n"
 			     , $tref->{modhier}, $modref->name);
+	    $fileref->printf("${indent} register %s* ts = %s;\n"
+			     , $modref->name, $tref->{nethier});
+	}
+	# Now do the signal
+	if ($tref->{identical_child} && !$tref->{identical}) {   # This code is reused by a child module.
+	    $fileref->printf("${indent}  _identcode[".$tref->{identical_child}."] = c;\n");
+	}
+	my $c = "c";
+	my $ket = "";
+	if ($tref->{identical}) {
+	    $c = "lc";
+	    $fileref->printf("${indent}  {int lc=_identcode[".$tref->{identical}."];\n"); 
+	    $ket .= "}";
 	}
 	if ($netref->array && !$tref->{ignore}) {
 	    $fileref->printf("${indent}  for (int i=0; i<%s; ++i) {\n"
 			     ,$netref->array);
 	    $indent .= "  ";
+	    $ket .= "}";
 	}
 
 	if ($tref->{ignore}) {
@@ -207,6 +263,7 @@ sub _write_tracer_init {
 	} else {
 	    $fileref->printf("${indent}  {");
 	}
+	$ket .= "}";
 	if ($netref->type eq "sc_clock") {
 	    $fileref->printf("const bool& tempClk=%s;\n", $accessor);
 	    $fileref->printf("${indent}   ");
@@ -216,22 +273,19 @@ sub _write_tracer_init {
 	my $arraynum = ($netref->array ? " i":"-1");
 	$fileref->printf("");
 	if ($width == 1) {
-	    $fileref->printf("vcdp->declBit  (c,\"%s\",%s,&(%s)"
+	    $fileref->printf("vcdp->declBit  (${c},\"%s\",%s,&(%s)"
 			     ,$netref->name, $arraynum, ${accessor});
 	} elsif ($width <= 32) {
-	    $fileref->printf("vcdp->declBus  (c,\"%s\",%s,&(%s),%d,%d"
+	    $fileref->printf("vcdp->declBus  (${c},\"%s\",%s,&(%s),%d,%d"
 			     ,$netref->name, $arraynum, ${accessor},$netref->msb, $netref->lsb);
 	} else {
-	    $fileref->printf("vcdp->declArray(c,\"%s\",%s,&(%s),%d,%d",
+	    $fileref->printf("vcdp->declArray(${c},\"%s\",%s,&(%s),%d,%d",
 			     ,$netref->name, $arraynum, ${accessor},$netref->msb, $netref->lsb);
 	}
-	$fileref->printf("); c+=%s;}",$tref->{code_inc});
+	$fileref->printf("); ${c}+=%s;$ket",$tref->{code_inc});
 	$fileref->printf(" // Is-a: %s\n", $netref->type);
-	if ($netref->array && !$tref->{ignore}) {
-	    $indent = "  "x$tref->{level};
-	    $fileref->printf("${indent}  }\n");
-	}
     }
+    $fileref->printf("  }\n");
     $fileref->print("}\n");
 }
 
@@ -250,12 +304,23 @@ sub _write_tracer_change {
 	$fileref->print("  int c=code;\n");
 	$fileref->print("  ${mod}* t=(${mod}*)userthis;\n");
     }
+    $fileref->printf("  {\n");
+    my $last_modhier = undef;
     foreach my $tref (@tracevars) {
+	my $modref = $tref->{modref};
 	my $netref = $tref->{netref};
 	next if $tref->{ignore};
+	next if $tref->{identical};
 	my $indent = "  "x$tref->{level};
 	my $accessor = $tref->{accessor};
 
+	if (!defined $last_modhier || $last_modhier ne $tref->{modhier}) {
+	    $last_modhier = $tref->{modhier};
+	    $fileref->printf("${indent}"."}{\n");
+	    $fileref->printf("${indent} register %s* ts = %s;\n"
+			     , $modref->name, $tref->{nethier});
+	}
+
 	if ($netref->array) {
 	    $fileref->printf("${indent}  for (int i=0; i<%s; ++i) {\n"
 			     ,$netref->array);
@@ -286,6 +351,7 @@ sub _write_tracer_change {
 	    $fileref->printf("${indent}  }\n");
 	}
     }
+    $fileref->printf("  }\n");
     $fileref->print("}\n");
 }
 
