@@ -1,4 +1,5 @@
 #!/usr/local/bin/perl -w
+# $Id: 9x_common_build.pl,v 1.5 2002/03/11 14:07:22 wsnyder Exp $
 # DESCRIPTION: Perl ExtUtils: Type 'make test' to test this package
 
 my $dir = "test_dir";
@@ -14,7 +15,7 @@ if ($Config{archname} !~ /linux/
     print "Skipping: Not linux with systemc installed\n";
     skip(1,1);
 } else {
-    run_system ("cd $dir && make -f ../example/Makefile_obj");
+    run_system ("cd $dir && make -j 3 -f ../example/Makefile_obj");
     ok(1);
 }
 
@@ -29,5 +30,5 @@ if (! -x "$dir/ex_main"
     run_system ("cd $dir && ./ex_main");
     ok(1);
     ok(-r "$dir/sim_sc.vcd");
-    ok(-r "$dir/sim_sp.vcd");
+    ok(-r "$dir/sim_sp.dump");
 }
